#!/usr/bin/env node
(function() {
  var Worker, app, cluster, createCoreImage, discardSignal, signalHandlers, stopProcess, terminateProcess;

  cluster = require('cluster');

  app = process.env.APP || process.argv[2];

  process.title = "zion " + (cluster.isMaster && 'master' || 'worker') + ": " + app;

  discardSignal = stopProcess = function(signal, workers) {};

  terminateProcess = createCoreImage = function(signal, workers) {
    var worker, _i, _len;
    for (_i = 0, _len = workers.length; _i < _len; _i++) {
      worker = workers[_i];
      worker.kill(true);
    }
    return process.exit(1);
  };

  signalHandlers = {
    SIGHUP: terminateProcess,
    SIGINT: terminateProcess,
    SIGQUIT: createCoreImage,
    SIGILL: createCoreImage,
    SIGTRAP: createCoreImage,
    SIGABRT: createCoreImage,
    SIGEMT: createCoreImage,
    SIGFPE: createCoreImage,
    SIGKILL: terminateProcess,
    SIGBUS: createCoreImage,
    SIGSEGV: createCoreImage,
    SIGSYS: createCoreImage,
    SIGPIPE: terminateProcess,
    SIGALRM: terminateProcess,
    SIGTERM: terminateProcess,
    SIGURG: discardSignal,
    SIGSTOP: stopProcess,
    SIGTSTP: stopProcess,
    SIGCONT: discardSignal,
    SIGCHLD: discardSignal,
    SIGTTIN: stopProcess,
    SIGTTOU: stopProcess,
    SIGIO: discardSignal,
    SIGXCPU: terminateProcess,
    SIGXFSZ: terminateProcess,
    SIGVTALRM: terminateProcess,
    SIGPROF: terminateProcess,
    SIGWINCH: discardSignal,
    SIGINFO: discardSignal,
    SIGUSR1: terminateProcess,
    SIGUSR2: terminateProcess
  };

  Worker = (function() {
    var getWorkerIndex, handler, signal, workers, _fn;

    workers = [];

    getWorkerIndex = function(proc) {
      var k, v, _len;
      for (k = 0, _len = workers.length; k < _len; k++) {
        v = workers[k];
        if (v.proc === proc) return k;
      }
    };

    cluster.on('death', function(proc) {
      var index;
      index = getWorkerIndex(proc);
      return workers[index].onDeath(proc, index);
    });

    _fn = function(signal, handler) {
      return process.on(signal, function() {
        return handler(signal, workers);
      });
    };
    for (signal in signalHandlers) {
      handler = signalHandlers[signal];
      _fn(signal, handler);
    }

    function Worker(msg) {
      this.msg = msg != null ? msg : {};
      workers.push(this);
      this.fork();
    }

    Worker.prototype.onDeath = function(proc, index) {
      if (proc.exitCode === 0 || this.isKilled) {
        return workers.splice(index, 1);
      } else {
        return this.fork();
      }
    };

    Worker.prototype.kill = function(permanent) {
      if (permanent == null) permanent = false;
      this.isKilled = permanent;
      return process.kill(this.proc.pid);
    };

    Worker.prototype.fork = function() {
      var proc;
      var _this = this;
      process.env.APP = app;
      this.proc = proc = cluster.fork();
      return proc.on('message', function(msg) {
        var worker, _i, _len, _results;
        if (msg.cmd === 'zion:online') {
          return proc.send(_this.msg);
        } else if (msg.cmd === 'zion:fork') {
          return new Worker(msg);
        } else if (msg.cmd === 'zion:restart') {
          _results = [];
          for (_i = 0, _len = workers.length; _i < _len; _i++) {
            worker = workers[_i];
            _results.push(process.kill(worker.proc.pid));
          }
          return _results;
        }
      });
    };

    return Worker;

  })();

  if (cluster.isMaster) {
    new Worker({
      cmd: 'zion:bootstrap'
    });
  } else {
    process.chdir(app);
    process.send({
      cmd: 'zion:online'
    });
    process.on('message', function(msg) {
      return require(app)(msg);
    });
  }

}).call(this);
